/****************************************************************************

    DRC: Digital Room Correction
    Copyright (C) 2002 Denis Sbragion

		This file is part of DRC, Digital Room Correction

		DRC  is  distributed  with  NO WARRANTY OF ANY KIND. No author or
		distributor accepts any responsibility for  the  consequences  of
		using  it,  or  for  whether  it serves any particular purpose or
		works at all, unless he or she says so in writing.  Refer to  the
		Aladdin Free Public License (the "License") for full details.

		Every copy of DRC must include a copy of the License, normally in
		a plain ASCII text file named PUBLIC. The License grants you  the
		right  to  copy, modify and redistribute DRC, but only under cer
		tain conditions described in the License.   Among  other  things,
		the License requires that the copyright notice and this notice be
		preserved on all copies.

		You can contact the author on Internet at the following address:

				d.sbragion@infotecna.it

		This program uses the parsecfg library from Yuuki  NINOMIYA.  De
		tails  on  this  library  can be found in the parsecfg.c and par
		secfg.h files.  Many thanks to Yuuki NINOMIYA for this useful li
		brary.

****************************************************************************/

/* Main file */

/* Inclusioni */
#include "drc.h"
#include "dsplib.h"
#include "dspwind.h"
#include "prefilt.h"
#include "baselib.h"
#include "level.h"
#include "parsecfg.h"
#include "convol.h"
#include "hd.h"
#include "toeplitz.h"
#include "fir.h"
#include <stdio.h>

/* Opzioni di configurazione */
CfgParmsType Cfg;

/* Definizione struttura file di configurazione */
cfgStruct CfgParmsDef[] = 
	{		
		/* Base configuration */
		{ "BCInFile",CFG_STRING,&Cfg.BCInFile },
		{ "BCInFileType",CFG_STRING,&Cfg.BCInFileType },
		{ "BCSampleRate",CFG_INT,&Cfg.BCSampleRate },		
		{ "BCImpulseCenterMode",CFG_STRING,&Cfg.BCImpulseCenterMode },
		{ "BCImpulseCenter",CFG_INT,&Cfg.BCImpulseCenter },
		{ "BCInitWindow",CFG_INT,&Cfg.BCInitWindow },
		{ "BCNormFactor",DRCCfgFloat,&Cfg.BCNormFactor },
		{ "BCNormType",CFG_STRING,&Cfg.BCNormType },

		/* Homomorphic Deconvolution */
		{ "HDMultExponent",CFG_INT,&Cfg.HDMultExponent },
		{ "HDMPNormFactor",DRCCfgFloat,&Cfg.HDMPNormFactor },
		{ "HDMPNormType",CFG_STRING,&Cfg.HDMPNormType },
		{ "HDMPOutFile",CFG_STRING,&Cfg.HDMPOutFile },
		{ "HDMPOutFileType",CFG_STRING,&Cfg.HDMPOutFileType },
		{ "HDEPNormFactor",DRCCfgFloat,&Cfg.HDEPNormFactor },
		{ "HDEPNormType",CFG_STRING,&Cfg.HDEPNormType },
		{ "HDEPOutFile",CFG_STRING,&Cfg.HDEPOutFile },
		{ "HDEPOutFileType",CFG_STRING,&Cfg.HDEPOutFileType },

		/* Minimum phase band windowing stage */		
		{ "MPInitWindow",CFG_INT,&Cfg.MPInitWindow },
		{ "MPFilterLen",CFG_INT,&Cfg.MPFilterLen },
		{ "MPBandSplit",CFG_INT,&Cfg.MPBandSplit },
		{ "MPStartFreq",DRCCfgFloat,&Cfg.MPStartFreq },
		{ "MPEndFreq",DRCCfgFloat,&Cfg.MPEndFreq },
		{ "MPWindowExponent",DRCCfgFloat,&Cfg.MPWindowExponent },
		{ "MPBWFinalWindow",CFG_INT,&Cfg.MPBWFinalWindow },
		{ "MPBWNormFactor",DRCCfgFloat,&Cfg.MPBWNormFactor },
		{ "MPBWNormType",CFG_STRING,&Cfg.MPBWNormType },
		{ "MPBWOutFile",CFG_STRING,&Cfg.MPBWOutFile },
		{ "MPBWOutFileType",CFG_STRING,&Cfg.MPBWOutFileType },

		/* Excess fase phase band windowing stage */
		{ "EPInitWindow",CFG_INT,&Cfg.EPInitWindow },
		{ "EPFilterLen",CFG_INT,&Cfg.EPFilterLen },
		{ "EPBandSplit",CFG_INT,&Cfg.EPBandSplit },
		{ "EPStartFreq",DRCCfgFloat,&Cfg.EPStartFreq },
		{ "EPEndFreq",DRCCfgFloat,&Cfg.EPEndFreq },
		{ "EPWindowExponent",DRCCfgFloat,&Cfg.EPWindowExponent },
		{ "EPBWFinalWindow",CFG_INT,&Cfg.EPBWFinalWindow },
		{ "EPBWFlatGain",DRCCfgFloat,&Cfg.EPBWFlatGain },
		{ "EPBWFGMultExponent",CFG_INT,&Cfg.EPBWFGMultExponent },
		{ "EPBWNormFactor",DRCCfgFloat,&Cfg.EPBWNormFactor },
		{ "EPBWNormType",CFG_STRING,&Cfg.EPBWNormType },
		{ "EPBWOutFile",CFG_STRING,&Cfg.EPBWOutFile },
		{ "EPBWOutFileType",CFG_STRING,&Cfg.EPBWOutFileType },

		/* Dip limiting stage */
		{ "DLMinGain",DRCCfgFloat,&Cfg.DLMinGain },
		{ "DLMultExponent",CFG_INT,&Cfg.DLMultExponent },

		/* Prefiltering completion stage */
		{ "PCOutWindow",CFG_INT,&Cfg.PCOutWindow },
		{ "PCNormFactor",DRCCfgFloat,&Cfg.PCNormFactor },
		{ "PCNormType",CFG_STRING,&Cfg.PCNormType },		
		{ "PCOutFile",CFG_STRING,&Cfg.PCOutFile },
		{ "PCOutFileType",CFG_STRING,&Cfg.PCOutFileType },

		/* Inversion stage */
		{ "ISDeltaDelay",CFG_INT,&Cfg.ISDeltaDelay },
		{ "ISOutWindow",CFG_INT,&Cfg.ISOutWindow },
		{ "ISNormFactor",DRCCfgFloat,&Cfg.ISNormFactor },
		{ "ISNormType",CFG_STRING,&Cfg.ISNormType },		
		{ "ISOutFile",CFG_STRING,&Cfg.ISOutFile },
		{ "ISOutFileType",CFG_STRING,&Cfg.ISOutFileType },

		/* Postfiltering stage */
		{ "PSFilterLen",CFG_INT,&Cfg.PSFilterLen },
		{ "PSNumPoints",CFG_INT,&Cfg.PSNumPoints },
		{ "PSPointsFile",CFG_STRING,&Cfg.PSPointsFile },		
		{ "PSOutWindow",CFG_INT,&Cfg.PSOutWindow },
		{ "PSNormFactor",DRCCfgFloat,&Cfg.PSNormFactor },
		{ "PSNormType",CFG_STRING,&Cfg.PSNormType },		
		{ "PSOutFile",CFG_STRING,&Cfg.PSOutFile },
		{ "PSOutFileType",CFG_STRING,&Cfg.PSOutFileType },

		/* Peak limiting stage */
		{ "PLMaxGain",DRCCfgFloat,&Cfg.PLMaxGain },
		{ "PLMultExponent",CFG_INT,&Cfg.PLMultExponent },
		{ "PLOutWindow",CFG_INT,&Cfg.PLOutWindow },
		{ "PLNormFactor",DRCCfgFloat,&Cfg.PLNormFactor },
		{ "PLNormType",CFG_STRING,&Cfg.PLNormType },		
		{ "PLOutFile",CFG_STRING,&Cfg.PLOutFile },
		{ "PLOutFileType",CFG_STRING,&Cfg.PLOutFileType },

		/* Chiusura lista parametri */
		{NULL, CFG_END, NULL}	
	};

int main(int argc, char * argv[])
	{
		/* Segnale in ingresso */
		DLReal * InSig;

		/* Componenti minimum phase e excess phase del segnale */
		DLReal * MPSig;
		DLReal * EPSig;

		/* Punto iniziale e dimenione finestrature */
		int WStart1;
		int WLen1;
		int WStart2;
		int WLen2;

		/* Calcolo finestra validazione parametri */
		int PWLen;

		/* Componenti MP e EP prefiltrate */
		DLReal * MPBWSig;
		int MPBWSigLen;
		DLReal * EPBWSig;
		int EPBWSigLen;

		/* Array convoluzione MP/EP */
		DLReal * MPEPSig;
		int MPEPSigLen;

		/* Array inversione impulso */
		DLReal * ISRevSig;
		DLReal * ISRevOut;

		/* Array postfiltratura */
		DLReal * PSFilterFreqs;
		DLComplex * PSFilterPoints;
		DLReal * PSFilter;
		DLReal * PSOutSig;

		/* Indice generico */
		int I;

		/* Messaggio iniziale */
		sputs("\nDRC 1.02.0001: Digital Room Correction");
		sputs("Copyright (C) 2002 Denis Sbragion");
		sputs("\nThis program is distributed under the terms of the AFPL License");
		sputs("and is provided to you as is, without any warranty.");
		sputs("Please read the file \"PUBLIC\" for details.");

		/* Controllo presenza argomenti */
		if (argc < 2)
			{				
				sputs("\nUsage: DRC DRCFile"); 				
				sputs("\nParameters:\n");
				sputs("  DRCFile: name of DRC configration file");
				sputs("  Refer to the manual and samples for file format\n");
				return 0;
			}		

		/* Recupera la configurazione */
		sputs("\nParsing configuration file...");
		if (cfgParse(argv[1],CfgParmsDef,CFG_SIMPLE) != CFG_NO_ERROR)
			{
				sputs("Configuration file parsing error.");
				return 1;
			}
		sputs("Parsing completed.");

		/* Controllo validit parametri */
		sputs("Configuration parameters check.");
		if (Cfg.BCInFile == NULL)
			{
				sputs("BC: No input file name supplied.");
				return 1;
			}
		if (Cfg.BCInFileType == NULL)
			{
				sputs("BC: No input file type supplied.");
				return 1;
			}
		if (Cfg.BCImpulseCenterMode == NULL)
			{
				sputs("BC: No impulse center mode supplied.");
				return 1;
			}
		if (Cfg.BCNormFactor > 0 && Cfg.BCNormType == NULL)
			{
				sputs("BC: No input normalization type supplied.");
				return 1;
			}		
		if (Cfg.BCInitWindow <= 0)
			{
				sputs("BC: No initial window supplied.");
				return 1;
			}		

		if (Cfg.HDMPNormFactor > 0 && Cfg.HDMPNormType == NULL)
			{
				sputs("HD: No MP normalization type supplied.");
				return 1;
			}
		if (Cfg.HDEPNormFactor > 0 && Cfg.HDEPNormType == NULL)
			{
				sputs("HD: No EP normalization type supplied.");
				return 1;
			}		

		if (Cfg.HDMPOutFile != NULL && Cfg.HDMPOutFileType == NULL)
			{
				sputs("HD: No MP output file type supplied.");
				return 1;
			}		
		if (Cfg.HDEPOutFile != NULL && Cfg.HDEPOutFileType == NULL)
			{
				sputs("HD: No EP output file type supplied.");
				return 1;
			}		

		if (Cfg.MPInitWindow <= 0)
			{
				sputs("MP: Mo MPInitWindow supplied.");
				return 1;
			}
		if (Cfg.MPInitWindow > 2 * Cfg.BCInitWindow)
			{
				sputs("MP: MPInitWindow can't be greater than 2 * BCInitWindow.");
				return 1;
			}
		if (Cfg.MPBWNormFactor > 0 && Cfg.MPBWNormType == NULL)
			{
				sputs("MP: No BW normalization type supplied.");
				return 1;
			}
		if (Cfg.MPBWOutFile != NULL && Cfg.MPBWOutFileType == NULL)
			{
				sputs("MP: No MPBW output file type supplied.");
				return 1;
			}		
		if (Cfg.MPBWFinalWindow > Cfg.MPInitWindow + Cfg.MPFilterLen - 1)
			{
				sputs("MP: MPBWFinalWindow can't be greater than MPInitWindow + MPFilterLen - 1.");
				return 1;
			}		
		if (Cfg.MPBWFinalWindow <= 0)
			PWLen = Cfg.MPBWFinalWindow;
		else
			PWLen = Cfg.MPInitWindow + Cfg.MPFilterLen - 1;

		if (Cfg.EPInitWindow <= 0)
			{
				sputs("MP: No EPInitWindow supplied.");
				return 1;
			}
		if (Cfg.EPInitWindow > Cfg.BCInitWindow)
			{
				sputs("MP: EPInitWindow can't be greater than BCInitWindow.");
				return 1;
			}
		if (Cfg.EPBWNormFactor > 0 && Cfg.EPBWNormType == NULL)
			{
				sputs("EP: No BW normalization type supplied.");
				return 1;
			}
		if (Cfg.EPBWOutFile != NULL && Cfg.EPBWOutFileType == NULL)
			{
				sputs("EP: No EPBW output file type supplied.");
				return 1;
			}		
		if (Cfg.EPBWFinalWindow > Cfg.EPInitWindow + Cfg.EPFilterLen - 1)
			{
				sputs("EP: EPBWFinalWindow can't be greater than EPInitWindow + EPFilterLen - 1.");
				return 1;
			}		
		if (Cfg.EPBWFinalWindow <= 0)
			PWLen += (Cfg.EPInitWindow + Cfg.EPFilterLen - 1) - 1;
		else
			PWLen += Cfg.EPBWFinalWindow - 1;

		if (Cfg.PCOutWindow <= 0)
			{
				sputs("PC: No PCOutWindow supplied.");
				return 1;
			}
		if (Cfg.PCNormFactor > 0 && Cfg.PCNormType == NULL)
			{
				sputs("PC: No normalization type supplied.");
				return 1;
			}
		if (Cfg.PCOutFile != NULL && Cfg.PCOutFileType == NULL)
			{
				sputs("PC: No output file type supplied.");
				return 1;
			}		
		if (Cfg.PCOutWindow > PWLen)
			{
				sputs("PC: PCOutWindow too big.");
				return 1;
			}		
		PWLen = Cfg.PCOutWindow;

		if (Cfg.ISNormFactor > 0 && Cfg.ISNormType == NULL)
			{
				sputs("IS: No normalization type supplied.");
				return 1;
			}
		if (Cfg.ISOutFile != NULL && Cfg.ISOutFileType == NULL)
			{
				sputs("IS: No output file type supplied.");
				return 1;
			}		
		if ((Cfg.PCOutWindow >= 0 && Cfg.ISDeltaDelay >= Cfg.PCOutWindow) || 
			(Cfg.PCOutWindow == 0 && Cfg.ISDeltaDelay >= PWLen))
			{
				sputs("IS: ISDeltaDelay too big.");
				return 1;
			}		
		if (Cfg.ISOutWindow > 0)
			PWLen = Cfg.ISOutWindow;

		if (Cfg.PSPointsFile == NULL)
			{
				sputs("PS: No correction point file supplied.");
				return 1;
			}		
		if (Cfg.PSNormFactor > 0 && Cfg.PSNormType == NULL)
			{
				sputs("PS: No normalization type supplied.");
				return 1;
			}
		if (Cfg.PSOutFile != NULL && Cfg.PSOutFileType == NULL)
			{
				sputs("PS: No output file type supplied.");
				return 1;
			}		

		if (Cfg.PLNormFactor > 0 && Cfg.PLNormType == NULL)
			{
				sputs("PL: No normalization type supplied.");
				return 1;
			}
		if (Cfg.PLOutFile != NULL && Cfg.PLOutFileType == NULL)
			{
				sputs("PL: No output file type supplied.");
				return 1;
			}		
		if (Cfg.PLOutWindow >= 0 && Cfg.PLOutWindow > PWLen)
			{
				sputs("PL: PLOutWindow too big.");
				return 1;
			}
		
		/* Controlla il tipo ricerca centro impulso */
		if (Cfg.BCImpulseCenterMode[0] == 'A')
			{
				/* Ricerca il centro impulso */
				sputs("Seeking impulse center...");
				Cfg.BCImpulseCenter = FindMaxPcm(Cfg.BCInFile,(IFileType) Cfg.BCInFileType[0]);
				if (Cfg.BCImpulseCenter < 0)
					return 1;
				printf("Impulse center found at sample %i.\n",Cfg.BCImpulseCenter);
				fflush(stdout);
			}

		/* Alloca l'array per il segnale in ingresso */
		sputs("Allocating input signal array.");
		InSig = new DLReal[Cfg.BCInitWindow];
		if (InSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Legge il file di ingresso */
		sputs("Reading input signal.");		
		if (ReadSignal(Cfg.BCInFile,InSig,Cfg.BCInitWindow,Cfg.BCImpulseCenter,
			(IFileType) Cfg.BCInFileType[0]) == False)
			{
				sputs("Error reading input signal.");
				return 1;
			}
		sputs("Input signal read.");

		/* Verifica se si deve effettuare rinormalizzazione */
		if (Cfg.BCNormFactor > 0)
			{
				sputs("Input signal normalization.");
				SigNormalize(InSig,Cfg.BCInitWindow,Cfg.BCNormFactor,
					(NormType) Cfg.BCNormType[0]);
			}

		/* Alloca gli array per la deconvoluzione omomorfa */
		sputs("Allocating homomorphic deconvolution arrays.");
		MPSig = new DLReal[2 * Cfg.BCInitWindow];
		if (MPSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}
		EPSig = new DLReal[Cfg.BCInitWindow];
		if (EPSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Azzera gli array */
		for (I = 0;I < 2 * Cfg.BCInitWindow;I++)
			MPSig[I] = 0;
		for (I = 0;I < Cfg.BCInitWindow;I++)
			EPSig[I] = 0;

		/* Effettua la deconvoluzione omomorfa*/
		sputs("Homomorphic deconvolution stage...");
		if (CepstrumHD(InSig,&MPSig[Cfg.BCInitWindow],EPSig,
			Cfg.BCInitWindow,Cfg.HDMultExponent) == False)
			{
				sputs("Homomorphic deconvolution failed.");
				return 1;
			}

		/* Verifica se si deve effettuare rinormalizzazione */
		if (Cfg.HDMPNormFactor > 0)
			{
				sputs("Minimum phase component normalization.");
				SigNormalize(MPSig,2 * Cfg.BCInitWindow,Cfg.HDMPNormFactor,
					(NormType) Cfg.HDMPNormType[0]);
			}
		if (Cfg.HDEPNormFactor > 0)
			{
				sputs("Excess phase component normalization.");
				SigNormalize(EPSig,Cfg.BCInitWindow,Cfg.HDEPNormFactor,
					(NormType) Cfg.HDEPNormType[0]);
			}

		/* Verifica se si deve salvare la componente MP */
		if (Cfg.HDMPOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving minimum phase component.");
				if (WriteSignal(Cfg.HDMPOutFile,&MPSig[Cfg.BCInitWindow],Cfg.BCInitWindow,
					(IFileType) Cfg.HDMPOutFileType[0]) == False)
					{
						sputs("Minimum phase component save failed.");
						return 1;
					}
			}

		/* Verifica se si deve salvare la componente EP */
		if (Cfg.HDEPOutFile != NULL)
			{				
				/* Salva la componente EP */
				sputs("Saving excess phase component.");
				if (WriteSignal(Cfg.HDEPOutFile,EPSig,Cfg.BCInitWindow,
					(IFileType) Cfg.HDEPOutFileType[0]) == False)
					{
						sputs("Excess phase component save failed.");
						return 1;
					}
			}

		/* Delloca il segnale di ingresso */
		delete InSig;

		/* Alloca l'array per il segnale MP prefiltrato */
		sputs("Allocating minimum phase component band windowing array.");
		MPBWSigLen = Cfg.MPInitWindow + Cfg.MPFilterLen - 1;
		MPBWSig = new DLReal[MPBWSigLen];		
		if (MPBWSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Azzera l'array */
		for (I = 0;I < MPBWSigLen;I++)
			MPBWSig[I] = 0;

		/* Calcola il punto iniziale finestra */
		WStart1 = ((2 * Cfg.BCInitWindow) - Cfg.MPInitWindow) / 2;

		/* Prefiltratura componente MP */
		sputs("Minimum phase component band windowing.");
		PreFilt(&MPSig[WStart1],Cfg.MPInitWindow,Cfg.MPFilterLen,
			Cfg.MPBandSplit,Cfg.MPWindowExponent,Cfg.BCSampleRate, 
			Cfg.MPStartFreq,Cfg.MPEndFreq,MPBWSig);

		/* Delloca la componente MP */
		delete MPSig;

		/* Verifica se si deve effettuare una finestratura finale */
		if (Cfg.MPBWFinalWindow > 0)
			{
				WStart1 = (MPBWSigLen - Cfg.MPBWFinalWindow) / 2;
				WLen1 = Cfg.MPBWFinalWindow;
				sputs("Minimum phase component final windowing.");
				BlackmanWindow(&MPBWSig[WStart1],WLen1);
			}
		else
			{
				WStart1 = 0;
				WLen1 = MPBWSigLen;
			}

		/* Verifica se si deve effettuare rinormalizzazione */
		if (Cfg.MPBWNormFactor > 0)
			{
				sputs("Minimum phase component normalization.");
				SigNormalize(&MPBWSig[WStart1],WLen1,Cfg.MPBWNormFactor,
					(NormType) Cfg.MPBWNormType[0]);
			}

		/* Verifica se si deve salvare la componente MP finestrata */
		if (Cfg.MPBWOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving minimum phase component.");
				if (WriteSignal(Cfg.MPBWOutFile,&MPBWSig[WStart1],WLen1,
					(IFileType) Cfg.MPBWOutFileType[0]) == False)
					{
						sputs("Minimum phase component save failed.");
						return 1;
					}
			}

		/* Alloca l'array per il segnale EP prefiltrato */
		sputs("Allocating excess phase component band windowing array.");
		EPBWSigLen = Cfg.EPInitWindow + Cfg.EPFilterLen - 1;
		EPBWSig = new DLReal[EPBWSigLen];		
		if (EPBWSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Azzera l'array */
		for (I = 0;I < EPBWSigLen;I++)
			EPBWSig[I] = 0;

		/* Calcola il punto iniziale finestra */
		WStart2 = (Cfg.BCInitWindow - Cfg.EPInitWindow) / 2;

		/* Prefiltratura componente EP */
		sputs("Excess phase component band windowing.");
		PreFilt(&EPSig[WStart2],Cfg.EPInitWindow,Cfg.EPFilterLen,
			Cfg.EPBandSplit,Cfg.EPWindowExponent,Cfg.BCSampleRate, 
			Cfg.EPStartFreq,Cfg.EPEndFreq,EPBWSig);

		/* Delloca la componente EP */
		delete EPSig;

		/* Verifica se si deve effettuare una finestratura finale */
		if (Cfg.EPBWFinalWindow > 0)
			{
				WStart2 = (EPBWSigLen - Cfg.EPBWFinalWindow) / 2;
				WLen2 = Cfg.EPBWFinalWindow;
				sputs("Excess phase component final windowing.");
				BlackmanWindow(&EPBWSig[WStart2],WLen2);
			}
		else
			{
				WStart1 = 0;
				WLen1 = MPBWSigLen;
			}

		/* Verifica se si deve effettuare riappianamento */
		if (Cfg.EPBWFlatGain > 0)
			{
				sputs("Excess phase component flattening.");
				NormFlat(&EPBWSig[WStart2],WLen2,Cfg.EPBWFlatGain,
					Cfg.EPBWFGMultExponent);
			}

		/* Verifica se si deve effettuare rinormalizzazione */
		if (Cfg.EPBWNormFactor > 0)
			{
				sputs("Excess phase component normalization.");
				SigNormalize(&EPBWSig[WStart2],WLen2,Cfg.EPBWNormFactor,
					(NormType) Cfg.EPBWNormType[0]);
			}

		/* Verifica se si deve salvare la componente EP finestrata */
		if (Cfg.EPBWOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving excess phase component.");
				if (WriteSignal(Cfg.EPBWOutFile,&EPBWSig[WStart2],WLen2,
					(IFileType) Cfg.EPBWOutFileType[0]) == False)
					{
						sputs("Excess phase component save failed.");
						return 1;
					}
			}

		/* Alloca l'array per la convoluzione MP/EP */
		sputs("Allocating MP/EP convolution array.");
		MPEPSigLen = WLen1 + WLen2 - 1;
		MPEPSig = new DLReal[MPEPSigLen];		
		if (MPEPSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Convoluzione MP/EP */
		sputs("MP/EP Convolution...");
		if (FftConvolve(&MPBWSig[WStart1],WLen1,&EPBWSig[WStart2],WLen2,MPEPSig) == False)
			{
				sputs("Convolution failed.");
				return 1;
			}

		/* Dealloca gli array MP/EP finestrati */
		delete MPBWSig;
		delete EPBWSig;

		/* Determina la dimensione della finestra di uscita */
		if (Cfg.PCOutWindow > 0)
			{
				WStart1 = (MPEPSigLen - Cfg.PCOutWindow) / 2;
				WLen1 = Cfg.PCOutWindow;
			}
		else
			{
				WStart1 = 0;
				WLen1 = MPEPSigLen;
			}

		/* Controlla se si deve effettuare il dip limiting */
		if (Cfg.DLMinGain > 0)
			{
				sputs("MP/EP dip limiting...");
				if (DipLimit(&MPEPSig[WStart1],WLen1,Cfg.DLMinGain,Cfg.DLMultExponent) == False)
					{
						sputs("Dip limiting failed.");
						return 1;
					}
			}

		/* Finestratura segnale risultante */
		if (Cfg.PCOutWindow > 0)
			{
				sputs("MP/EP signal windowing.");
				BlackmanWindow(&MPEPSig[WStart1],WLen1);
			}

		/* Normalizzazione segnale risultante */
		if (Cfg.PCNormFactor > 0)
			{
				sputs("MP/EP normalization.");
				SigNormalize(&MPEPSig[WStart1],WLen1,Cfg.PCNormFactor,
					(NormType) Cfg.PCNormType[0]);
			}

		/* Verifica se si deve salvare il segnale prefinestrato */
		if (Cfg.PCOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving MP/EP signal.");
				if (WriteSignal(Cfg.PCOutFile,&MPEPSig[WStart1],WLen1,
					(IFileType) Cfg.PCOutFileType[0]) == False)
					{
						sputs("MP/EP signal save failed.");
						return 1;
					}
			}

		/* Alloca l'array per l'inversione segnale */
		sputs("Allocating delay/reverse array.");
		ISRevSig = new DLReal[WLen1];
		if (ISRevSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Inversione e ritardo segnale */
		sputs("Signal delay/reverse.");
		for (I = 0;I < WLen1;I++)
			ISRevSig[WLen1 - (I + 1)] =
				MPEPSig[WStart1 + ((I + Cfg.ISDeltaDelay) % WLen1)];

		/* Calcolo autocorrelazione e setup inversione */
		sputs("Autocorrelation computation and reverse setup...");
		if (AutoCorrelation(&MPEPSig[WStart1],WLen1) == False)
			{
				sputs("Autocorrelation computation failed.");
				return 1;
			}
		for (I = WLen1 / 2; I < WLen1; I++)
			MPEPSig[WStart1 + I] = 0;

		/* Alloca l'array per l'inversione segnale */
		sputs("Allocating inversion array.");
		ISRevOut = new DLReal[WLen1];
		if (ISRevOut == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Effettua l'inversione del segnale */
		sputs("Toeplitz least square inversion...");
		if (ToeplitzSolve(&MPEPSig[WStart1],ISRevSig,ISRevOut,WLen1) != 0)
			{
				sputs("Inversion failed.");
				return(1);
			}

		/* Dealloca gli array */
		delete MPEPSig;
		delete ISRevSig;

		/* Finestratura segnale risultante */
		if (Cfg.ISOutWindow > 0)
			{
				sputs("Inverted signal windowing.");
				WStart2 = (WLen1 - Cfg.ISOutWindow) / 2;
				WLen2 = Cfg.ISOutWindow;
				BlackmanWindow(&ISRevOut[WStart2],WLen2);
			}
		else
			{
				WStart2 = 0;
				WLen2 = WLen1;
			}

		/* Normalizzazione segnale risultante */
		if (Cfg.ISNormFactor > 0)
			{
				sputs("Inverted signal normalization.");
				SigNormalize(&ISRevOut[WStart2],WLen2,Cfg.ISNormFactor,
					(NormType) Cfg.ISNormType[0]);
			}

		/* Verifica se si deve salvare il segnale invertito */
		if (Cfg.ISOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving inverted signal.");
				if (WriteSignal(Cfg.ISOutFile,&ISRevOut[WStart2],WLen2,
					(IFileType) Cfg.ISOutFileType[0]) == False)
					{
						sputs("Inverted signal save failed.");
						return 1;
					}
			}

		/* Alloca gli array per la generazione del postfiltro */
		sputs("Allocating post filter arrays.");
		PSFilterFreqs = new DLReal[Cfg.PSNumPoints];
		if (PSFilterFreqs == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}
		PSFilterPoints = new DLComplex[Cfg.PSNumPoints];
		if (PSFilterPoints == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}
		PSFilter = new DLReal[Cfg.PSFilterLen];
		if (PSFilter == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}
		for (I = 0; I < Cfg.PSFilterLen; I++)
			PSFilter[I] = 0;
		WLen1 = Cfg.PSFilterLen + WLen2 - 1;
		PSOutSig = new DLReal[WLen1];
		if (PSOutSig == NULL)
			{
				sputs("Memory allocation failed.");
				return 1;
			}

		/* Legge i punti del filtro */
		if (ReadPoints(Cfg.PSPointsFile,PSFilterFreqs,PSFilterPoints,
			Cfg.PSNumPoints,Cfg.BCSampleRate) == False)
			{
				sputs("Correction point file input failed.");
				return 1;
			}

		/* Calcola il filtro */
		sputs("FIR Filter computation...");
		if (GenericFir(PSFilter,Cfg.PSFilterLen,
			PSFilterFreqs,PSFilterPoints,Cfg.PSNumPoints) == False)
			{
				sputs("FIR Filter computation failed.");
				return 1;
			}
		BlackmanWindow(PSFilter,Cfg.PSFilterLen); 

		/* Convoluzione filtro segnale */
		sputs("FIR Filter convolution...");
		if (FftConvolve(&ISRevOut[WStart2],WLen2,PSFilter,Cfg.PSFilterLen,PSOutSig) == False)
			{
				perror("Convolution failed.");
				return 1;
			}		

		/* Deallocazione array */
		delete PSFilterFreqs;
		delete PSFilterPoints;
		delete ISRevOut;
		delete PSFilter;		

		/* Determina la dimensione della finestra di uscita */
		if (Cfg.PSOutWindow > 0)
			{
				WStart2 = (WLen1 - Cfg.PCOutWindow) / 2;
				WLen2 = Cfg.PSOutWindow;
				sputs("Postfiltered signal windowing.");
				BlackmanWindow(&PSOutSig[WStart2],WLen2);
			}
		else
			{
				WStart2 = 0;
				WLen2 = WLen1;
			}

		/* Normalizzazione segnale risultante */
		if (Cfg.PSNormFactor > 0)
			{
				sputs("Postfiltered signal normalization.");
				SigNormalize(&PSOutSig[WStart2],WLen2,Cfg.PSNormFactor,
					(NormType) Cfg.PSNormType[0]);
			}

		/* Verifica se si deve salvare il segnale postfiltrato */
		if (Cfg.PSOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving postfiltered signal.");
				if (WriteSignal(Cfg.PSOutFile,&PSOutSig[WStart2],WLen2,
					(IFileType) Cfg.PSOutFileType[0]) == False)
					{
						sputs("Postfiltered signal save failed.");
						return 1;
					}
			}

		/* Controlla se si deve effettuare il peak limiting */
		if (Cfg.PLMaxGain > 0)
			{
				sputs("Peak limiting...");
				if (PeakLimit(&PSOutSig[WStart2],WLen2,
					Cfg.PLMaxGain,Cfg.PLMultExponent) == False)
					{
						sputs("Peak limiting failed.");
						return 1;
					}
			}

		/* Determina la dimensione della finestra di uscita */
		if (Cfg.PLOutWindow > 0)
			{
				WStart2 = (WLen1 - Cfg.PLOutWindow) / 2;
				WLen2 = Cfg.PLOutWindow;
				sputs("Postfiltered signal final windowing.");
				BlackmanWindow(&PSOutSig[WStart2],WLen2);
			}

		/* Normalizzazione segnale risultante */
		if (Cfg.PLNormFactor > 0)
			{
				sputs("Postfiltered signal normalization.");
				SigNormalize(&PSOutSig[WStart2],WLen2,Cfg.PLNormFactor,
					(NormType) Cfg.PLNormType[0]);
			}

		/* Verifica se si deve salvare il segnale postfiltrato */
		if (Cfg.PLOutFile != NULL)
			{				
				/* Salva la componente MP */
				sputs("Saving postfiltered signal.");
				if (WriteSignal(Cfg.PLOutFile,&PSOutSig[WStart2],WLen2,
					(IFileType) Cfg.PLOutFileType[0]) == False)
					{
						sputs("Postfiltered signal save failed.");
						return 1;
					}
			}

		/* Esecuzione completata */
		sputs("Execution completed.");
		return 0;
	}
