/****************************************************************************

    DRC: Digital Room Correction
    Copyright (C) 2002 Denis Sbragion

		This file is part of DRC, Digital Room Correction

		DRC  is  distributed  with  NO WARRANTY OF ANY KIND. No author or
		distributor accepts any responsibility for  the  consequences  of
		using  it,  or  for  whether  it serves any particular purpose or
		works at all, unless he or she says so in writing.  Refer to  the
		Aladdin Free Public License (the "License") for full details.

		Every copy of DRC must include a copy of the License, normally in
		a plain ASCII text file named PUBLIC. The License grants you  the
		right  to  copy, modify and redistribute DRC, but only under cer
		tain conditions described in the License.   Among  other  things,
		the License requires that the copyright notice and this notice be
		preserved on all copies.

		You can contact the author on Internet at the following address:

				d.sbragion@infotecna.it

		This program uses the parsecfg library from Yuuki  NINOMIYA.  De
		tails  on  this  library  can be found in the parsecfg.c and par
		secfg.h files.  Many thanks to Yuuki NINOMIYA for this useful li
		brary.

****************************************************************************/

/* Header generale DRC */

#ifndef DRC_h
	#define DRC_h

	#include "parsecfg.h"

	/* Tipo floating point usato per le elaborazioni */
	#define DRCFloat float

	/* Tipo corrispondente per l'importazione di valori di 
	configurazione */
	#define DRCCfgFloat CFG_FLOAT

	/* Valore minimo del tipo float in uso */
	#define  DRCMinFloat 1.175494351e-38F 

	/* Tipo floating point usato per l'importazione e 
	l'esportazione dei segnali floating point */
	#define DRCFileFloat float

	/* Tipo intero usato per l'importazione e 
	l'esportazione dei segnali interi */
	#define DRCFileInt short int

	/* Struttura definizione configurazione */
	/* Prefissi variabili
		BC = Base Configuration
		HD = Homomorphic Deconvolution
		MP = Minimum phase band windowing stage
		EP = Excess fase phase band windowing stage
		DL = Dip limiting stage
		PC = Prefiltering completion stage		
		IS = Inversion stage
		PS = Postfiltering stage
		PL = Peak limiting stage
	*/
	typedef struct
		{
			/* Base configuration */
			char * BCInFile;
			char * BCInFileType;
			int BCSampleRate;					
			char * BCImpulseCenterMode;
			int BCImpulseCenter;
			int BCInitWindow;
			DRCFloat BCNormFactor;
			char * BCNormType;

			/* Homomorphic Deconvolution */
			int HDMultExponent;
			DRCFloat HDMPNormFactor;
			char * HDMPNormType;
			char * HDMPOutFile;
			char * HDMPOutFileType;
			DRCFloat HDEPNormFactor;
			char * HDEPNormType;
			char * HDEPOutFile;
			char * HDEPOutFileType;			

			/* Minimum phase band windowing stage */			
			int MPInitWindow;
			int MPFilterLen;			
			int MPBandSplit;
			DRCFloat MPStartFreq;
			DRCFloat MPEndFreq;
			DRCFloat MPWindowExponent;
			int MPBWFinalWindow;
			DRCFloat MPBWNormFactor;
			char * MPBWNormType;
			char * MPBWOutFile;
			char * MPBWOutFileType;			

			/* Excess fase phase band windowing stage */			
			int EPInitWindow;
			int EPFilterLen;			
			int EPBandSplit;
			DRCFloat EPStartFreq;
			DRCFloat EPEndFreq;
			DRCFloat EPWindowExponent;	
			int EPBWFinalWindow;
			DRCFloat EPBWFlatGain;
			int EPBWFGMultExponent;
			DRCFloat EPBWNormFactor;
			char * EPBWNormType;
			char * EPBWOutFile;
			char * EPBWOutFileType;

			/* Dip limiting stage */
			DRCFloat DLMinGain;
			int DLMultExponent;

			/* Prefiltering completion stage */
			int PCOutWindow;
			DRCFloat PCNormFactor;
			char * PCNormType;			
			char * PCOutFile;
			char * PCOutFileType;			

			/* Inversion stage */
			int ISDeltaDelay;
			int ISOutWindow;
			DRCFloat ISNormFactor;
			char * ISNormType;
			char * ISOutFile;
			char * ISOutFileType;

			/* Postfiltering stage */
			int PSFilterLen;
			int PSNumPoints;
			char * PSPointsFile;
			int PSOutWindow;
			DRCFloat PSNormFactor;
			char * PSNormType;
			char * PSOutFile;
			char * PSOutFileType;

			/* Peak limiting stage */
			DRCFloat PLMaxGain;
			int PLMultExponent;
			int PLOutWindow;
			DRCFloat PLNormFactor;
			char * PLNormType;
			char * PLOutFile;
			char * PLOutFileType;
		}
	CfgParmsType;
#endif
