/****************************************************************************

    DRC: Digital Room Correction
    Copyright (C) 2002 Denis Sbragion

		This file is part of DRC, Digital Room Correction

		DRC  is  distributed  with  NO WARRANTY OF ANY KIND. No author or
		distributor accepts any responsibility for  the  consequences  of
		using  it,  or  for  whether  it serves any particular purpose or
		works at all, unless he or she says so in writing.  Refer to  the
		Aladdin Free Public License (the "License") for full details.

		Every copy of DRC must include a copy of the License, normally in
		a plain ASCII text file named PUBLIC. The License grants you  the
		right  to  copy, modify and redistribute DRC, but only under cer
		tain conditions described in the License.   Among  other  things,
		the License requires that the copyright notice and this notice be
		preserved on all copies.

		You can contact the author on Internet at the following address:

				d.sbragion@infotecna.it

		This program uses the parsecfg library from Yuuki  NINOMIYA.  De
		tails  on  this  library  can be found in the parsecfg.c and par
		secfg.h files.  Many thanks to Yuuki NINOMIYA for this useful li
		brary.

****************************************************************************/

/****************************************************************************
  Progetto    : DSP Library.
  File        : DSPWind.cpp
  Autore      : Sbragion Denis
  Descrizione : Funzioni per vari tipi di finestratura.
  Revisioni   :
  16/10/93    : Prima stesura.
****************************************************************************/

#include "dspwind.h"

void HammingWindow(DLReal * A,unsigned int Size)
  {
    unsigned int I,Half = Size/2;
    DLReal C;

    for(I = 0;I < Half;I++)
      {
        C = (DLReal) (0.54-0.46*cos(2*M_PI*I/(Size-1)));
        A[I] *= C;
        A[Size-I-1] *= C;
      }
  }

void HanningWindow(DLReal * A,unsigned int Size)
  {
    unsigned int I,Half = Size/2;
    DLReal C;

    for(I = 0;I < Half;I++)
      {
        C = (DLReal) (0.5*(1-cos(2*M_PI*I/(Size-1))));
        A[I] *= C;
        A[Size-I-1] *= C;
      }
  }

void BlackmanWindow(DLReal * A,unsigned int Size)
  {
    unsigned int I,Half = Size/2;
    DLReal C;

    for(I = 0;I < Half;I++)
      {
        C = (DLReal) (0.42-0.5*cos(2*M_PI*I/(Size-1))+0.08*cos(4*M_PI*I/(Size-1)));
        A[I] *= C;
        A[Size-I-1] *= C;
      }
  }

void BartlettWindow(DLReal * A,unsigned int Size)
  {
    unsigned int I,Half = Size/2;
    DLReal C,R = ((DLReal) Size-1)/2;

    for(I = 0;I < Half;I++)
      {
        C = I/R;
        A[I] *= C;
        A[Size-I-1] *= C;
      }
  }

void BlackmanHarrisWindow(DLReal * A,unsigned int Size)
	{
		unsigned int I,Half = Size/2;
    DLReal C;

    for(I = 0;I < Half;I++)
      {
        C = (DLReal) (0.35875-0.48829*cos(2*M_PI*I/(Size-1))+0.14128*cos(4*M_PI*I/(Size-1))+0.01168*cos(6*M_PI*I/(Size-1)));
        A[I] *= C;
        A[Size-I-1] *= C;
      }
	}

void ExponentialWindow(DLReal * A,unsigned int Size,DLReal P)
  {
    unsigned int I,Half = Size/2;
    DLReal C,X;

    P = (DLReal) (sqrt(P));

    for(I = 0;I < Half;I++)
      {
        X = (P*(Half-I))/Half;
        C = (DLReal) (exp(-X*X));
        A[I] *= C;
        A[Size-I-1] *= C;
      }
  }

void Ones(DLReal * A,unsigned int Size)
  {
    unsigned int I;

    for(I = 0;I < Size;I++)
      A[I] = 1;
  }

/***************************************************************************/
