/****************************************************************************

    DRC: Digital Room Correction
    Copyright (C) 2002 Denis Sbragion

		This file is part of DRC, Digital Room Correction

		DRC  is  distributed  with  NO WARRANTY OF ANY KIND. No author or
		distributor accepts any responsibility for  the  consequences  of
		using  it,  or  for  whether  it serves any particular purpose or
		works at all, unless he or she says so in writing.  Refer to  the
		Aladdin Free Public License (the "License") for full details.

		Every copy of DRC must include a copy of the License, normally in
		a plain ASCII text file named PUBLIC. The License grants you  the
		right  to  copy, modify and redistribute DRC, but only under cer
		tain conditions described in the License.   Among  other  things,
		the License requires that the copyright notice and this notice be
		preserved on all copies.

		You can contact the author on Internet at the following address:

				d.sbragion@infotecna.it

		This program uses the parsecfg library from Yuuki  NINOMIYA.  De
		tails  on  this  library  can be found in the parsecfg.c and par
		secfg.h files.  Many thanks to Yuuki NINOMIYA for this useful li
		brary.

****************************************************************************/

/****************************************************************************
  Progetto    : DSP Library.
  File        : Fir.h
  Autore      : Sbragion Denis
  Descrizione : Funzioni per il calcolo di filtri fir.
  Revisioni   :
  16/10/93    : Prima stesura.
****************************************************************************/

#ifndef Fir_h
  #define Fir_h

  #include "boolean.h"
  #include "dsplib.h"

  // Ritornano i filtri fir a fase lineare di ordine Order corrispondenti
  // al nome della funzione. Tutti i filtri sono ricavati con la sola
  // finestra rettangolare.
  // Per i filtri HighPass e BandStop Order deve essere dispari.
  // Se non lo e` viene calcolato un filtro con Order-1 coefficienti
  // trascurando l' ultimo che viene posto a 0.
  // Order deve comunque essere maggiore o uguale a 2.
  void LowPassFir(DLReal * Filter,unsigned int Order,DLReal Freq);
  void HighPassFir(DLReal * Filter,unsigned int Order,DLReal Freq);
  void BandPassFir(DLReal * Filter,unsigned int Order,DLReal Low,
    DLReal High);
  void BandStopFir(DLReal * Filter,unsigned int Order,DLReal Low,
    DLReal High);

  // Ritorna il trasformatore di Hilbert di ordine Order ricavato con la
  // sola finestra rettangolare.
  // Order deve essere dispari altrimenti l' ultimo coefficiente viene
  // ignorato e posto uguale a 0. Inoltre Order deve essere maggiore di 2.
  void HilbertFir(DLReal * Filter,unsigned int Order);

  // Genera un filtro Fir con risposta generica tramite IFft e
  // finestratura. L' ordine Order deve essere maggiore o uguale a 2.
  // Il filtro e` ricavato con la sola finestra rettangolare. Si ricordi
  // che i filtri di ordine dispari presentano generalmente un
  // comportamento piu` regolare.
  // F rappresenta l' array delle frequenze su cui costruire la risposta
  // del  filtro. Deve essere in ordine crescente con il primo
  // coefficiente 0 e l' ultimo, corrispondente a meta` della frequenza
  // di campionamento, uguale a 1. Abs(G[I]) e Arg(G[I]) rappresentano
  // guadagno e fase desiderati nei corrispondenti punti F[I].
  // Ritorna False in caso di errori.
  // Se Is e` 0 l' array per l' interpolazione intermedia viene preso
  // con dimensione pari alla piu` piccola potenza di 2 che supera Order,
  // altrimenti viene preso con dimensione Is. Is non deve comunque essere
  // inferiore a Order. Per avere una elevata precisione nella
  // ricostruzione della risposta utilizzare un elevato valore di Is.
  // Se Is e` una potenza di 2 il calcolo risulta molto piu` rapido.
  Boolean GenericFir(DLReal * Filter,unsigned int Order,DLReal * F,
    DLComplex * G,unsigned int Np,unsigned int Is = 0);

#endif

/***************************************************************************/
