/****************************************************************************

    DRC: Digital Room Correction
    Copyright (C) 2002 Denis Sbragion

		This file is part of DRC, Digital Room Correction

		DRC  is  distributed  with  NO WARRANTY OF ANY KIND. No author or
		distributor accepts any responsibility for  the  consequences  of
		using  it,  or  for  whether  it serves any particular purpose or
		works at all, unless he or she says so in writing.  Refer to  the
		Aladdin Free Public License (the "License") for full details.

		Every copy of DRC must include a copy of the License, normally in
		a plain ASCII text file named PUBLIC. The License grants you  the
		right  to  copy, modify and redistribute DRC, but only under cer
		tain conditions described in the License.   Among  other  things,
		the License requires that the copyright notice and this notice be
		preserved on all copies.

		You can contact the author on Internet at the following address:

				d.sbragion@infotecna.it

		This program uses the parsecfg library from Yuuki  NINOMIYA.  De
		tails  on  this  library  can be found in the parsecfg.c and par
		secfg.h files.  Many thanks to Yuuki NINOMIYA for this useful li
		brary.

****************************************************************************/

/* Deconvoluzione omomorfa */

/* Inclusioni */
#include "hd.h"
#include "fft.h"
#include "baselib.h"
#include <math.h>

/* Effettua la deconvoluzione omomorfa del segnale In */
Boolean CepstrumHD(const DLReal * In, DLReal * MPOut, DLReal * EPOut,
	const int N, const int PFactor)
	{
		DLComplex * FFTArray1;
		DLComplex * FFTArray2;
		DLComplex * FFTArray3;
		int FS;
		int I;

		/* Calcola la potenza di due superiore a N */
		for(FS = 1;FS <= N;FS <<= 1);
		FS *= 1 << PFactor;

		/* Alloca gli array per l'FFT */
		if ((FFTArray1 = new DLComplex[FS]) == NULL)
			return False;
		if ((FFTArray2 = new DLComplex[FS]) == NULL)
			return False;
		if ((FFTArray3 = new DLComplex[FS]) == NULL)
			return False;
			
		/* Copia l'array sorgente in quello temporaneo */
		for (I = 0;I < N;I++)
			FFTArray1[I] = In[I];

		/* Azzera la parte rimanente */
		for (I = N;I < FS;I++)
			FFTArray1[I] = 0;

		/* Trasforma l'array risultante */
		Radix2Fft(FFTArray1,FS);

		/* Calcola i valori per il cepstrum */
		for (I = 0;I < FS;I++)
			if (std::abs<DLReal>(FFTArray1[I]) <= 0)
				{
					sputs("Warning: log limit reached in cepstrum computation.");
					FFTArray2[I] = DRCMinFloat;
				}
			else
				FFTArray2[I] = std::log<DLReal>(std::abs<DLReal>(FFTArray1[I]));

		/* Calcola il cepstrum */
		Radix2IFft(FFTArray2,FS);

		/* Finestra il cepstrum */
		for (I = 1; I < FS/2;I++)
			FFTArray2[I] *= 2;
		for (I = FS/2 + 1; I < FS;I++)
			FFTArray2[I] = 0;

		/* Calcola la trsformat del cepstrum finestrato */
		Radix2Fft(FFTArray2,FS);

		/* Effettua il calcolo dell'esponenziale */
		for (I = 0;I < FS;I++)
			FFTArray2[I] = std::exp<DLReal>(FFTArray2[I]);

		/* Determina la trasformata della parte excess phase */
		for (I = 0;I < FS;I++)
			FFTArray3[I] = FFTArray1[I] / FFTArray2[I];

		/* Determina la risposta del sistema a fase minima */
		Radix2IFft(FFTArray2,FS);

		/* Copia il risultato nell'array destinazione */
		for (I = 0;I < N;I++)
			MPOut[I] = std::real<DLReal>(FFTArray2[I]);

		/* Determina la risposta del sistema excess phase */
		Radix2IFft(FFTArray3,FS);

		/* Copia il risultato nell'array destinazione */
		for (I = 0;I < N;I++)
			EPOut[I] = std::real<DLReal>(FFTArray3[I]);

		/* Dealloca gli array */
		delete FFTArray1;
		delete FFTArray2;
		delete FFTArray3;

		/* Operazione completata */
		return True;
	}
