/****************************************************************************

    DRC: Digital Room Correction
    Copyright (C) 2002 Denis Sbragion

		This file is part of DRC, Digital Room Correction

		DRC  is  distributed  with  NO WARRANTY OF ANY KIND. No author or
		distributor accepts any responsibility for  the  consequences  of
		using  it,  or  for  whether  it serves any particular purpose or
		works at all, unless he or she says so in writing.  Refer to  the
		Aladdin Free Public License (the "License") for full details.

		Every copy of DRC must include a copy of the License, normally in
		a plain ASCII text file named PUBLIC. The License grants you  the
		right  to  copy, modify and redistribute DRC, but only under cer
		tain conditions described in the License.   Among  other  things,
		the License requires that the copyright notice and this notice be
		preserved on all copies.

		You can contact the author on Internet at the following address:

				d.sbragion@infotecna.it

		This program uses the parsecfg library from Yuuki  NINOMIYA.  De
		tails  on  this  library  can be found in the parsecfg.c and par
		secfg.h files.  Many thanks to Yuuki NINOMIYA for this useful li
		brary.

****************************************************************************/

/* Prefiltratura a bande di un segnale */

#include <stdio.h>
#include "prefilt.h"
#include "convol.h"
#include "fir.h"
#include "dspwind.h"

/* Prefiltratura a bande di un segnale */
void PreFilt(const DLReal * InImp, const int IBS, const int FilterLen,
	const int BandSplit, const DLReal WindowExponent, const int SampleFreq, 
	const DLReal StartFreq, const DLReal EndFreq, DLReal * OutImp)
	{
		/* Array risultati parziali */
		DLReal * COut;

		/* Array filtro FIR */
		DLReal * FIRFilter;

		/* Array finestratura segnale di ingresso */
		DLReal * CIn;

		/* Array temporanei convoluzione */
		DLComplex * CA;
		DLComplex * CB;
		
		/* Numero banda corrente */
		int Band;

		/* Estremi della banda. */
		DLReal BLow;
		DLReal BHigh;

		/* Larghezza banda */
		DLReal BWidth;

		/* Inizio e fine filtratura */
		DLReal FilterBegin;
		DLReal FilterEnd;

  	/* Posizione finestra su segnale */
		int WStart;
		int WLen;
		int I;
		int OBS;

		/* Calcola la dimensione del blocco in uscita dalla convoluzione */
		OBS = IBS + FilterLen - 1;

		/* Calcola inizio e fine della prefiltratura */
		FilterBegin = (2 * StartFreq) / SampleFreq;
		FilterEnd = (2 * EndFreq) / SampleFreq;

		/* Alloca gli array temporanei */
		COut = new DLReal[OBS];
		FIRFilter = new DLReal[FilterLen];
		CIn = new DLReal[IBS];

		/* Calcola la dimensione degli array di appoggio convoluzione */ 
		if (FilterLen < IBS)
			for (I = 1; I <= FilterLen; I <<= 1);
		else
			for (I = 1; I <= IBS; I <<= 1);
		I *= 2;

		/* Alloca gli array di appoggio convoluzione */
		CA = new DLComplex[I];
		CB = new DLComplex[I];

		/* Imposta i parametri iniziali */
		Band = 0;
		BLow = 0;
		BHigh = 0;
		BWidth = (DLReal) pow(2,1.0/BandSplit);

		/* Ciclo sulle bande */
		while (BHigh < FilterEnd)
			{
				/* Calcola gli estremi banda */
				if (Band == 0)
					BLow = 0;
				else
					BLow = (DLReal) (FilterBegin * pow(BWidth,Band - 1)); 

				BHigh = (DLReal) (FilterBegin * pow(BWidth,Band));
				if (BHigh > FilterEnd)
					BHigh = 1.0;

				/* Calcola l'intervallo di finestratura */
				if (Band == 0)
					WLen = IBS;
				else
					WLen = (int) (1 + ceil((IBS * pow(FilterBegin / BHigh,WindowExponent)) / 2) * 2);
				WStart = (IBS - WLen) / 2;              
				
				printf("Band: %3d, %7.1f - %7.1f Hz, FIR, ", (int) Band,
					(double) (BLow * SampleFreq) / 2, (double) (BHigh * SampleFreq) / 2);
				fflush(stdout);				
				
				printf("wind: %6d, ", (int) WLen);
				fflush(stdout);				

				/* Effettua la finestratura del segnale */
				for (I = 0; I < IBS; I++)
					CIn[I] = InImp[I];
				BlackmanWindow(&CIn[WStart],WLen);
				for (I = 0; I < WStart; I++)
					CIn[I] = 0;
				for (I = WStart + WLen; I < IBS; I++)
					CIn[I] = 0;

				/* Calcola il filtro passa banda */
				BandPassFir(FIRFilter,FilterLen,BLow,BHigh);
				BlackmanWindow(FIRFilter,FilterLen);				

				printf("conv, ");
				fflush(stdout);

				/* Effettua la convoluzione tra segnale e filtro */
				FftConvolve(FIRFilter,FilterLen,CIn,IBS,COut,CA,CB);				
								
				printf("sum, ");
				fflush(stdout);

				/* Aggiunge il segnale al segnale risultante */
				for (I = 0; I < OBS; I++)
					OutImp[I] += COut[I];

				printf("done.\n");
				fflush(stdout);

				Band++;
			}

		/* Dealloca gli array intermedi */
		delete COut;
		delete FIRFilter;
		delete CA;
		delete CB;
	}
